// when animating on canvas, it is best to use requestAnimationFrame instead of setTimeout or setInterval
// not supported in all browsers though and sometimes needs a prefix, so we need a shim

// now we will setup our basic variables for the demo
var FireworkScanvas,FireworkSctx,FireworkScw,FireworkSch,FireworkSfireworks,FireworkSparticles,FireworkStimerTotal,FireworkStimerTick;

// now we are going to setup our function placeholders for the entire demo

// get a random number within a range
function random( min, max ) {
	return Math.random() * ( max - min ) + min;
}

// calculate the distance between two points
function calculateDistance( p1x, p1y, p2x, p2y ) {
	var xDistance = p1x - p2x,
			yDistance = p1y - p2y;
	return Math.sqrt( Math.pow( xDistance, 2 ) + Math.pow( yDistance, 2 ) );
}

// create firework
function Firework( sx, sy, tx, ty ) {
	// actual coordinates
	this.x = sx;
	this.y = sy;
	// starting coordinates
	this.sx = sx;
	this.sy = sy;
	// target coordinates
	this.tx = tx;
	this.ty = ty;
	// distance from starting point to target
	this.distanceToTarget = calculateDistance( sx, sy, tx, ty );
	this.distanceTraveled = 0;
	// track the past coordinates of each firework to create a trail effect, increase the coordinate count to create more prominent trails
	this.coordinates = [];
	this.coordinateCount = 5;
	// populate initial coordinate collection with the current coordinates
	while( this.coordinateCount-- ) {
		this.coordinates.push( [ this.x, this.y ] );
	}
	this.angle = Math.atan2( ty - sy, tx - sx );
	this.speed = 5;
	this.acceleration = 1.1;
	this.brightness = random( 50, 70 );
	// circle target indicator radius
	this.targetRadius = 1;
	//firework color
	this.hue=random(0,2000);
}

// update firework
Firework.prototype.update = function( index ) {
	// remove last item in coordinates array
	this.coordinates.pop();
	// add current coordinates to the start of the array
	this.coordinates.unshift( [ this.x, this.y ] );
	
	// cycle the circle target indicator radius
	if( this.targetRadius < 8 ) {
		this.targetRadius += 0.3;
	} else {
		this.targetRadius = 1;
	}
	
	// speed up the firework
	this.speed *= this.acceleration;
	
	// get the current velocities based on angle and speed
	var vx = Math.cos( this.angle ) * this.speed,
			vy = Math.sin( this.angle ) * this.speed;
	// how far will the firework have traveled with velocities applied?
	this.distanceTraveled = calculateDistance( this.sx, this.sy, this.x + vx, this.y + vy );
	
	// if the distance traveled, including velocities, is greater than the initial distance to the target, then the target has been reached
	if( this.distanceTraveled >= this.distanceToTarget ) {
		createParticles( this.tx, this.ty ,this.hue);
		// remove the firework, use the index passed into the update function to determine which to remove
		FireworkSfireworks.splice( index, 1 );
	} else {
		// target not reached, keep traveling
		this.x += vx;
		this.y += vy;
	}
}

// draw firework
Firework.prototype.draw = function() {
	FireworkSctx.beginPath();
	FireworkSctx.lineWidth=game.UI.scaleVal;
	// move to the last tracked coordinate in the set, then draw a line to the current x and y
	FireworkSctx.moveTo( this.coordinates[ this.coordinates.length - 1][ 0 ], this.coordinates[ this.coordinates.length - 1][ 1 ] );
	FireworkSctx.lineTo( this.x, this.y );
	FireworkSctx.strokeStyle = 'hsl(' + this.hue + ', 100%, ' + this.brightness + '%)';
	FireworkSctx.stroke();
	
	//FireworkSctx.beginPath();
	// draw the target for this firework with a pulsing circle
	//FireworkSctx.arc( this.tx, this.ty, this.targetRadius, 0, Math.PI * 2 );
	//FireworkSctx.stroke();
}

// create particle
function Particle( x, y , hueV) {
	this.x = x;
	this.y = y;
	// track the past coordinates of each particle to create a trail effect, increase the coordinate count to create more prominent trails
	this.coordinates = [];
	this.coordinateCount = 5;
	while( this.coordinateCount-- ) {
		this.coordinates.push( [ this.x, this.y ] );
	}
	// set a random angle in all possible directions, in radians
	this.angle = random( 0, Math.PI * 2 );
	this.speed = random( 1, 10 )*game.UI.scaleVal;
	// friction will slow the particle down
	this.friction = 0.95;
	// gravity will be applied and pull the particle down
	this.gravity = 1*game.UI.scaleVal;
	// set the hue to a random number +-20 of the overall hue variable
	this.hue = random( hueV - 20, hueV + 20 );
//this.hue=random(0,2000);
	this.brightness = random( 50, 80 );
	this.alpha = 1;
	// set how fast the particle fades out
	this.decay = random( 0.015, 0.03 );
}

// update particle
Particle.prototype.update = function( index ) {
	// remove last item in coordinates array
	this.coordinates.pop();
	// add current coordinates to the start of the array
	this.coordinates.unshift( [ this.x, this.y ] );
	// slow down the particle
	this.speed *= this.friction;
	// apply velocity
	this.x += Math.cos( this.angle ) * this.speed;
	this.y += Math.sin( this.angle ) * this.speed + this.gravity;
	// fade out the particle
	this.alpha -= this.decay;
	
	// remove the particle once the alpha is low enough, based on the passed in index
	if( this.alpha <= this.decay ) {
		FireworkSparticles.splice( index, 1 );
	}
}

// draw particle
Particle.prototype.draw = function() {
	FireworkSctx. beginPath();
	FireworkSctx.lineWidth=game.UI.scaleVal;
	// move to the last tracked coordinates in the set, then draw a line to the current x and y
	FireworkSctx.moveTo( this.coordinates[ this.coordinates.length - 1 ][ 0 ], this.coordinates[ this.coordinates.length - 1 ][ 1 ] );
	FireworkSctx.lineTo( this.x, this.y );
	FireworkSctx.strokeStyle = 'hsla(' + this.hue + ', 100%, ' + this.brightness + '%, ' + this.alpha + ')';
	FireworkSctx.stroke();
}

// create particle group/explosion
function createParticles( x, y , hue) {
	// increase the particle count for a bigger explosion, beware of the canvas performance hit with the increased particles though
	var particleCount = 50;
	while( particleCount-- ) {
		FireworkSparticles.push( new Particle( x, y , hue ) );
	}
}

function FireworksStart(){
	FireworkScanvas = document.getElementById( 'FireworkScanvas' );
	FireworkSctx = FireworkScanvas.getContext( '2d' );
	FireworkScw = FireworkScanvas.width;
	FireworkSch = FireworkScanvas.height;
	FireworkSfireworks = [];
	FireworkSparticles = [];
	FireworkStimerTotal = 5;
	FireworkStimerTick = 0;
	FireworkSmaxCount = 40;
	FireworkScurrentCount = 0;
}
// main demo loop
function FireworkSloop() {
	// normally, clearRect() would be used to clear the canvas
	// we want to create a trailing effect though
	// setting the composite operation to destination-out will allow us to clear the canvas at a specific opacity, rather than wiping it entirely
	FireworkSctx.globalCompositeOperation = 'destination-out';
	// decrease the alpha property to create more prominent trails
	FireworkSctx.fillStyle = 'rgba(0, 0, 0, 0.5)';
	FireworkSctx.fillRect( 0, 0, FireworkScw, FireworkSch );
	// change the composite operation back to our main mode
	// lighter creates bright highlight points as the fireworks and particles overlap each other
	FireworkSctx.globalCompositeOperation = 'lighter';
	
	// loop over each firework, draw it, update it
	var i = FireworkSfireworks.length;
	while( i-- ) {
		FireworkSfireworks[ i ].draw();
		FireworkSfireworks[ i ].update( i );
	}
	
	// loop over each particle, draw it, update it
	var i = FireworkSparticles.length;
	while( i-- ) {
		FireworkSparticles[ i ].draw();
		FireworkSparticles[ i ].update( i );
	}

	//stop launching after a preset count
	if(FireworkSmaxCount && FireworkScurrentCount>=FireworkSmaxCount)return;

	// launch fireworks automatically to random coordinates
	if( FireworkStimerTick >= FireworkStimerTotal ) {
		// start the firework at the bottom middle of the screen, then set the random target coordinates, the random y coordinates will be set within the range of the top half of the screen
		FireworkSfireworks.push( new Firework( FireworkScw / 2, FireworkSch, random( 0, FireworkScw ), random( 0, FireworkSch / 2 ) ) );
		FireworkStimerTick = 0;
		FireworkScurrentCount++;
	} else {
		FireworkStimerTick++;
	}
}
